```jenkinsfile
library 'std-jsl@master'

pipeline {
  agent none
  options {
    disableConcurrentBuilds()
  }
  environment{
    BUILD_TIMESTAMP = "0"

    CONFIGMAP_NAME = "gitlab"
    CONFIGMAP_NAMESPACE = "jenkins"
    CONFIGMAP_FILENAME = "${CONFIGMAP_NAME}.yaml"

    GIT_API_VERSION = "/api/v4"
    GIT_BRANCH = "master"
    GIT_FILE_NAME = "apollo.properties"
    GIT_HOST = "gitlab.ph.seabank.io"
    GIT_REPO_ID = "835"
    GIT_SCHEMA = "https"
    GIT_SHARE_PATH = "share"
    GIT_TOKEN = "bjUc56KSyKUXxSdMp9LK"
    GIT_APPID_FILENAME = "appid"

    KUBE_CLUSTER = "kubernetes"
    KUBE_CREDENTIAL = "dev-k8s"

    PROPS = ""
  }
  stages {
    stage('Init') {
      steps {
        script {
          BUILD_TIMESTAMP = "${currentBuild.startTimeInMillis}"
          CONFIGMAP_NAME = "gitlab-${params.MODULE}-${BUILD_TIMESTAMP}"
          KUBE_CLUSTER = determineCluster()
          KUBE_CREDENTIAL = determineCredential()
        }
      }
    }
    stage('Pull Config') {
      agent {
        label "nonlive"
      }
      steps {
        script {
          try {
            def prop = ""
            def response = httpRequest contentType: 'APPLICATION_JSON',
                validResponseCodes: "100:399", 
                url: "${GIT_SCHEMA}://${GIT_HOST}${GIT_API_VERSION}/projects/${GIT_REPO_ID}/repository/files/${params.VERSION}%2F${params.MODULE}%2F${params.ENVIRONMENT}%2F${params.CLUSTER}%2F${GIT_FILE_NAME}?ref=${GIT_BRANCH}",
                customHeaders: [[maskValue: true, name: 'PRIVATE-TOKEN', value: "${GIT_TOKEN}"]]
            def respBody = readJSON text: response.content
            prop = new String(respBody.content.decodeBase64(), 'UTF-8')
            PROPS = prop
          } catch(e) {
              println e
          }
        }
      }
    }
    stage('DeployConfig') {
      agent {
        label "nonlive"
      }
      steps {
        script {
          def yamlContent = [:]
          yamlContent.apiVersion = "v1"
          yamlContent.kind = "ConfigMap"
          yamlContent.metadata = [:]
          yamlContent.metadata.name = "${CONFIGMAP_NAME}"
          yamlContent.metadata.namespace = "${CONFIGMAP_NAMESPACE}"
          yamlContent.data = [:]
          yamlContent.data["${GIT_FILE_NAME}"] = PROPS

          writeYaml file: "${CONFIGMAP_FILENAME}", data: yamlContent, overwrite: true, charset: 'UTF-8'

          kubernetesDeploy(kubeconfigId: "${KUBE_CREDENTIAL}",
            configs: "${CONFIGMAP_FILENAME}",
            enableConfigSubstitution: false,
          )
        }
      }
    }
    stage('Update Apollo') {
      agent {
        kubernetes {
          cloud KUBE_CLUSTER
          yaml """\
            apiVersion: v1
            kind: Pod
            metadata:
              labels:
                some-label: some-label-value
            spec:
              containers:
              - name: jnlp
                image: harbor.ph.seabank.io/seabank/inbound-agent:4.3-4
              - name: apollo
                image: harbor.ph.seabank.io/seabank/apollo:v2.1.1
                imagePullPolicy: Always
                args:
                  - sleep
                  - -s
                  - 3600
                tty: true
                volumeMounts: 
                  - name: apollo
                    mountPath: /conf/config_${params.ENVIRONMENT}.yaml
                    subPath: config_${params.ENVIRONMENT}.yaml
                  - name: gitlab
                    mountPath: /conf/${GIT_FILE_NAME}
                    subPath: ${GIT_FILE_NAME}
              volumes:
                - name: apollo
                  configMap:
                    name: apollo
                    items:
                      - key: config_${params.ENVIRONMENT}.yaml
                        path: config_${params.ENVIRONMENT}.yaml
                - name: gitlab
                  configMap:
                    name: ${CONFIGMAP_NAME}
                    items:
                      - key: ${GIT_FILE_NAME}
                        path: ${GIT_FILE_NAME}
          """.stripIndent()
        }
      }
      steps {
        container('apollo') {
          sh "/apollo update -c ${params.CLUSTER} -m ${params.MODULE} -e ${params.ENVIRONMENT} -v ${params.VERSION} --config /conf/config_${params.ENVIRONMENT}.yaml"
        }
      }
    }
  }
  post{
      success{
          node('nonlive') {
              script{
                kubernetesDeploy(kubeconfigId: "${KUBE_CREDENTIAL}",
                  configs: "${CONFIGMAP_FILENAME}",
                  enableConfigSubstitution: false,
                  deleteResource: true,
                )
            }
          }
      }
      always{
          node('nonlive') {
              script{
                  wrap([$class: 'BuildUser']) {
                    currentBuild.displayName = "#${BUILD_NUMBER}-${params.MODULE}-${params.ENVIRONMENT}-${params.CLUSTER}"
                    currentBuild.description = "<div><li>VERSION: <strong>${params.VERSION}</strong></li><li>USER: <strong>${BUILD_USER_ID}</strong></li></div>"
                  }
                  try {
                    def prop = ""
                    def response = httpRequest contentType: 'APPLICATION_JSON',
                        validResponseCodes: "100:399", 
                        url: "${GIT_SCHEMA}://${GIT_HOST}${GIT_API_VERSION}/projects/${GIT_REPO_ID}/repository/tree",
                        customHeaders: [[maskValue: true, name: 'PRIVATE-TOKEN', value: "${GIT_TOKEN}"]]
                    def respBody = readJSON text: response.content

                    def appIdResponse = httpRequest contentType: 'APPLICATION_JSON',
                        validResponseCodes: "100:399", 
                        url: "${GIT_SCHEMA}://${GIT_HOST}${GIT_API_VERSION}/projects/${GIT_REPO_ID}/repository/files/${GIT_APPID_FILENAME}?ref=${GIT_BRANCH}",
                        customHeaders: [[maskValue: true, name: 'PRIVATE-TOKEN', value: "${GIT_TOKEN}"]]
                    def appIdRespBody = readJSON text: appIdResponse.content
                    def appIdStr = new String(appIdRespBody.content.decodeBase64(), 'UTF-8')
                    def appIdJson = readJSON text: appIdStr

                    apollo.set_params(respBody, appIdJson)

                  } catch(e) {
                    println e
                  }
              }
          }
      }
  }
}

def determineCluster() {
  switch(params.ENVIRONMENT) {
    case "dev1":
      return "dev"
    case "dev2":
      return "dev"
    case "sit1":
      return "dev"
    case "sit2":
      return "dev"
    case "uat1":
      return "core-uat"
    case "uat2":
      return "core-uat"
    case "staging":
      return "core-uat"
    case "staging-gray":
      return "core-uat"
    case "live":
      return "core-live-k8s"
    case "gray":
      return "core-live-k8s"
  }
}

def determineCredential() {
  switch(params.ENVIRONMENT) {
    case "dev1":
      return "dev-k8s"
    case "dev2":
      return "dev-k8s"
    case "sit1":
      return "dev-k8s"
    case "sit2":
      return "dev-k8s"
    case "uat1":
      return "core-uat-k8s"
    case "uat2":
      return "core-uat-k8s"
    case "staging":
      return "core-uat-k8s"
    case "staging-gray":
      return "core-uat-k8s"
    case "live":
      return "live-core-k8s"
    case "gray":
      return "live-core-k8s"
  }
}
```