#!/usr/bin/python3
# -*- coding: utf-8
import sys
import getopt
import subprocess

HARBOR_URL = "harbor.ph.seabank.io"


def exec_cmd(cmd):
    print("[INFO] ========start exec cmd============")
    print(f"[INFO] cmd: {cmd}")
    retcode, output = subprocess.getstatusoutput(cmd)
    print("[INFO] cmd retcode:%s" % retcode)
    print(output)
    if retcode > 0:
        print("[ERROR] cmd failed. exit.")
        print("[INFO] ========end exec cmd============")
        exit(retcode)
    print("[INFO] ========end exec cmd============")
    return output


def execute_job(NAMESPACE, PROJECT):
    print("[INFO] ================execute_job===================")
    job_yaml = f'''
---
  apiVersion: batch/v1
  kind: Job
  metadata: 
    name: init-{PROJECT}-mysql8-data
  spec: 
    template: 
      metadata: 
        name: init-{PROJECT}-mysql8-data
        labels: 
          middleware: {PROJECT}-mysql8-init
      spec: 
        priorityClassName: mysql-priority
        restartPolicy: Never
        containers: 
          - name: init-{PROJECT}-mysql8-data
            image: {HARBOR_URL}/ops/ubuntu-20-04-mysql8-basedata:v3.1
            command: 
              - "/usr/bin/cp"
            args: 
              - "-rp"
              - "/data/mysql"
              - "/data/mysql_tmp"
              - "/tmp/mysql"
            resources: 
              requests: 
                cpu: "4"
                memory: 8Gi
              limits: 
                cpu: "4"
                memory: 8Gi
            volumeMounts: 
              - name: mysql8-data
                mountPath: /tmp/mysql
        volumes: 
          - name: mysql8-data
            hostPath: 
              path: /data/mysql8/{NAMESPACE}/{PROJECT}
              type: DirectoryOrCreate    
    '''
    # print(job_yaml)
    job_name = "int_mysql8_job.yaml"
    print("[INFO] ===> start create %s" % job_name)
    with open(job_name, "w") as f:
        f.write(job_yaml)

    init_mysql_cmd = f"kubectl apply -f ./{job_name} -n {NAMESPACE}"
    exec_cmd(init_mysql_cmd)

    wait_init_mysql_cmd = f"kubectl wait --for=condition=complete job/init-{PROJECT}-mysql8-data  --timeout=120s -n " \
                          f"{NAMESPACE}"
    exec_cmd(wait_init_mysql_cmd)


def execute_deployment(NAMESPACE, PROJECT):
    print("[INFO] ==============execute_deployment=====================")
    print("[INFO] ===>  start get nodename")
    cmd = "kubectl get po -l middleware=%s-mysql8-init -owide --no-headers -n %s | awk '{print $7}'|tail -n1" % (
        PROJECT, NAMESPACE)
    output = exec_cmd(cmd)
    NODENAME = output.strip()
    if "-node" not in NODENAME:
        print("[ERROR] not found mysql8-init pod and node. exit")
        exit(1)
    deploy_yaml = f'''
---
  apiVersion: v1
  kind: Service
  metadata: 
    name: {PROJECT}-mysql8
  spec: 
    selector: 
      middleware: {PROJECT}-mysql8
    ports: 
      - name: port6606
        port: 6606
        targetPort: 6606
---
  apiVersion: apps/v1
  kind: Deployment
  metadata: 
    name: {PROJECT}-mysql8
  spec: 
    minReadySeconds: 3
    revisionHistoryLimit: 10
    strategy:
      rollingUpdate:
        maxSurge: 50%
        maxUnavailable: 50%
      type: RollingUpdate
    replicas: 1
    selector: 
      matchLabels: 
        middleware: {PROJECT}-mysql8
    template: 
      metadata: 
        name: {PROJECT}-mysql8
        labels: 
          middleware: {PROJECT}-mysql8
      spec: 
        priorityClassName: mysql-priority
        affinity: 
          nodeAffinity: 
            requiredDuringSchedulingIgnoredDuringExecution: 
              nodeSelectorTerms: 
                - matchExpressions: 
                  - key: kubernetes.io/hostname
                    operator: In
                    values: 
                      - {NODENAME}
        containers: 
          - name: {PROJECT}-mysql8
            image: {HARBOR_URL}/ops/ubuntu-20-04-mysql8-baseimg:v3.4-mem
            env: 
              - name: POD_IP
                valueFrom: 
                  fieldRef: 
                    fieldPath: status.podIP
            ports: 
              - name: port6606
                containerPort: 6606
            resources: 
              requests: 
                cpu: "4"
                memory: 8Gi
              limits: 
                cpu: "4"
                memory: 8Gi
            volumeMounts: 
              - name: mysql8-data
                mountPath: /data
        volumes: 
          - name: mysql8-data
            hostPath: 
              path: /data/mysql8/{NAMESPACE}/{PROJECT}
              type: DirectoryOrCreate
    '''
    # print(deploy_yaml)
    deploy_name = "mysql8.yaml"
    print("[INFO] start create %s" % deploy_name)
    with open(deploy_name, "w") as f:
        f.write(deploy_yaml)

    mysql_cmd = f"kubectl apply -f ./{deploy_name} -n {NAMESPACE}"
    exec_cmd(mysql_cmd)

    rollout_status_cmd = f"kubectl rollout status deploy {PROJECT}-mysql8 --timeout=120s -n {NAMESPACE}"
    exec_cmd(rollout_status_cmd)
    print("[INFO] deployment mysql is ready now.")

    get_svc_cmd = f"kubectl get svc  {PROJECT}-mysql8  -n {NAMESPACE};kubectl get pod -n {NAMESPACE} -o wide | grep " \
                  f"{PROJECT}-mysql8"
    exec_cmd(get_svc_cmd)


def get_svc_info(NAMESPACE, PROJECT):
    get_svc_cmd = f"kubectl get svc  {PROJECT}-mysql8  -n {NAMESPACE};kubectl get pod -n {NAMESPACE} -o wide | grep " \
                  f"{PROJECT}-mysql8"
    exec_cmd(get_svc_cmd)


def main(argv):
    print(argv)
    NAMESPACE = ''
    PROJECT = ''
    SKIP = ''
    GET = ''
    try:
        opts, args = getopt.getopt(argv, "hn:p:s:g:", ["namespace=", "project=", "skip=", "get="])
        print( 'opts', opts)
        print('args',args)
    except getopt.GetoptError:
        print(
            'install_pod_mysql.py -n <namespace> -p <project> -s \n eg: install_pod_mysql.py -n core-sit1-my -p '
            'payment')
        exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print(
                'usage: install_pod_mysql.py -n <namespace> -p <project>  \n eg: install_pod_mysql.py -n core-sit1-my '
                '-p payment')
            exit(2)
        elif opt in ("-n", "--namespace"):
            NAMESPACE = arg
        elif opt in ("-p", "--project"):
            PROJECT = arg
        elif opt in ("-s", "--skip"):
            SKIP = arg
        elif opt in ("-g", "--get"):
            GET = arg

    if not NAMESPACE or not PROJECT:
        print(
            'please input namespace and project. \n install_pod_mysql.py -n <namespace> -p <project>  \n eg: '
            'install_pod_mysql.py -n core-sit1-my -p payment')
        exit(2)

    if GET:
        get_svc_info(NAMESPACE, PROJECT)
        exit(0)

    if not SKIP:
        execute_job(NAMESPACE, PROJECT)
    execute_deployment(NAMESPACE, PROJECT)


if __name__ == "__main__":
    main(sys.argv[1:])
